/*
 * bq25792.h
 *
 *  Created on: Jun 28, 2021
 *      Author: ProDe
 */

#ifndef INC_BQ25792_H_
#define INC_BQ25792_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "i2c.h"
#include "stdbool.h"

#define ADDR_BQ25792  	(0x6B << 1) // Use 8-bit address

#define BQ25792_EN_CHARGE			BIT(5)

#define BQ25792_OFF					0
#define BQ25792_ON					1

#define	MINIMAL_SYSTEM_VOLTAGE	 	0x00
#define	CHARGE_VOLTAGE_LIMIT_MSB 	0x01
#define	CHARGE_VOLTAGE_LIMIT_LSB 	0x02
#define CHARGE_CURRENT_LIMIT_MSB 	0x03
#define CHARGE_CURRENT_LIMIT_LSB 	0x04
#define INPUT_VOLTAGE_LIMIT 	 	0x05
#define	INPUT_CURRENT_LIMIT_MSB 	0x06
#define	INPUT_CURRENT_LIMIT_LSB 	0x07
#define PRECHARGE_CONTROL 		 	0x08
#define TERMINATION_CONTROL 		0x09
#define RECHARGE_CONTROL 	 		0x0A
#define VOTG_REGULATION 		 	0x0B
#define IOTG_REGULATION 	 		0x0D
#define TIMER_CONTROL 			 	0x0E
#define CHARGER_CONTROL_0 	 		0x0F
#define CHARGER_CONTROL_1 	 		0x10
#define CHARGER_CONTROL_2 	 		0x11
#define CHARGER_CONTROL_3 	 		0x12
#define CHARGER_CONTROL_4 	 		0x13
#define CHARGER_CONTROL_5 	 		0x14
#define RESERVED_0 					0x15
#define TEMPERATURE_CONTROL 		0x16
#define NTC_CONTROL_0 		 		0x17
#define NTC_CONTROL_1 		 		0x18
#define ICO_CURRENT_LIMIT 			0x19
#define CHARGER_STATUS_0 			0x1B
#define CHARGER_STATUS_1 			0x1C
#define CHARGER_STATUS_2 			0x1D
#define CHARGER_STATUS_3 			0x1E
#define CHARGER_STATUS_4 			0x1F
#define FAULT_STATUS_0 	 			0x20
#define FAULT_STATUS_1 	 			0x21
#define CHARGER_FLAG_0 	 			0x22
#define CHARGER_FLAG_1 	 			0x23
#define CHARGER_FLAG_2 	 			0x24
#define CHARGER_FLAG_3 	 			0x25
#define	FAULT_FLAG_0			 	0x26
#define	FAULT_FLAG_1			 	0x27
#define CHARGER_MASK_0 	 			0x28
#define CHARGER_MASK_1 	 			0x29
#define CHARGER_MASK_2 	 			0x2A
#define CHARGER_MASK_3 	 			0x2B
#define FAULT_MASK_0 		 		0x2C
#define FAULT_MASK_1 		 		0x2D
#define ADC_CONTROL 	 			0x2E
#define ADC_FUNCTION_DISABLE_0 		0x2F
#define ADC_FUNCTION_DISABLE_1 		0x30
#define IBUS_ADC_MSB 			 	0x31
#define IBUS_ADC_LSB 			 	0x32
#define IBAT_ADC_MSB 			 	0x33
#define IBAT_ADC_LSB			 	0x34
#define VBUS_ADC_MSB 			 	0x35
#define VBUS_ADC_LSB 			 	0x36
#define VAC1_ADC 			 		0x37
#define VAC2_ADC 			 		0x39
#define VBAT_ADC_MSB 			 	0x3B
#define VBAT_ADC_LSB			 	0x3C
#define VSYS_ADC 			 		0x3D
#define TS_ADC 			 			0x3F
#define	TDIE_ADC				 	0x41
#define	DP_ADC					 	0x43
#define	DM_ADC 					 	0x45
#define	DPDM_DRIVER 			 	0x47
#define	PART_INFORMATION 		 	0x48

#define BITS_PER_LONG 32

#define BIT(nr)						(1UL << (nr))
#define GENMASK(h, l) 				(((~0UL) << (l)) & (~0UL >> (BITS_PER_LONG - 1 - (h))))


#define BQ25792_DIS_STAT			BIT(4)
#define BQ25792_EN_CHARGE			BIT(5)


#define BQ25792_PG_FLAG_INT			BIT(7)
#define BQ25792_VBUS_FLAG_INT		BIT(4)

#define BQ25792_CHRG_EN				BIT(5)
#define BQ25792_VBUS_PRESENT 		BIT(0)
#define BQ25792_PG_STAT 			BIT(3)
#define BQ25792_TEMP_MASK 			GENMASK(3, 0)
#define BQ25792_NOT_CHRGING     	0
#define BQ25792_VBUS_STAT_MSK   	GENMASK(4, 1)
#define BQ25792_CHG_STAT_MSK    	GENMASK(7, 5)
#define BQ25792_VBAT_PRESENT 		BIT(0)

typedef enum
{
	Not_Charging 					= 0,
	Trickle_Charge 					= BIT(5),
	Pre_Charge 						= BIT(6),
	Fast_Charge 					= (BIT(5) | BIT(6)),
	Taper_Charge 					= BIT(7),
	Top_Off_Timer_Active_Charging 	= (BIT(6) | BIT(7)),
	Charge_Termination_Done 		= (BIT(5) | BIT(6) | BIT(7)),
}BQ25792_Charge_State;

typedef struct BQ25792_state
{
	uint8_t online;
	BQ25792_Charge_State chrg_status;
	uint8_t chrg_type;
	uint8_t vbat_present;
	uint8_t temp;
	uint8_t vbus_status;
	uint8_t fault_0;
	uint8_t fault_1;
	uint8_t charger_flag_1;
	uint32_t vbat_adc;
	uint32_t vbus_adc;
	uint32_t ibus_adc;
	uint32_t ibat_adc;
} BQ25792_state;

typedef struct BQ25792_Device
{
	I2C_HandleTypeDef* hi2c;
	uint8_t addr;

	BQ25792_state state;
} BQ25792_Device;

extern BQ25792_Device BQ25792;

void bq25792_Init(I2C_HandleTypeDef*, struct BQ25792_Device**);
int bq25792_update_state();
int bq25792_set_charging(uint8_t);
int bq25792_set_vindpm(uint16_t);
#ifdef __cplusplus
}
#endif

#endif /* INC_BQ25792_H_ */
