/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.h
  * @brief          : Header for main.c file.
  *                   This file contains the common defines of the application.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2021 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under Ultimate Liberty license
  * SLA0044, the "License"; You may not use this file except in compliance with
  * the License. You may obtain a copy of the License at:
  *                             www.st.com/SLA0044
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MAIN_H
#define __MAIN_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "stm32l5xx_hal.h"

#include "stm32l5xx_ll_ucpd.h"
#include "stm32l5xx_ll_bus.h"
#include "stm32l5xx_ll_cortex.h"
#include "stm32l5xx_ll_rcc.h"
#include "stm32l5xx_ll_system.h"
#include "stm32l5xx_ll_utils.h"
#include "stm32l5xx_ll_pwr.h"
#include "stm32l5xx_ll_gpio.h"
#include "stm32l5xx_ll_dma.h"

#include "stm32l5xx_ll_exti.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "adc.h"
#include "bq25792.h"

#ifdef DEBUG
	#include "stdio.h"
#endif
/* USER CODE END Includes */

/* Exported types ------------------------------------------------------------*/
/* USER CODE BEGIN ET */
#define SPI_RX_SIZE 8
#define SPI_TX_SIZE 64
#define UART_RX_SIZE 15

enum eADC { ADC1_PowerStage, ADC1_VUSB, ADC1_CNT };

extern struct DeviceState
{
	struct BQ25792_Device* Charger;
	struct BQ40Z80_Device* Gauge;

	union
	{
		struct OptoSense_SPI
		{
			enum OptoSense_BUS
			{
			  OptoSense_BUS_Self,
			  OptoSense_BUS_CHARGER,
			  OptoSense_BUS_GAUGE,
			} BUS;

			union Param1
			{
				uint8_t Raw;
				uint8_t Register;
				enum OptoSense_Self_CMD
				{
				  OptoSense_DISABLE_POWERSTAGE,
				  OptoSense_ENABLE_POWERSTAGE,
				  OptoSense_SET_POWERSTAGE,
				  OptoSense_SystemReset,
				  OptoSense_BatteryShutdown,
				} Self_CMD;
			}P1;

			uint8_t Data;

			uint8_t Length;

			//Not Used But We Need Them To Access I2C While SPI Transaction Is In Progress... Half Completed CB
			uint8_t Dummy[4];
		} Packet;
		uint8_t RAW[SPI_RX_SIZE];
	}SPI_RxBuffer;

	uint8_t SPI_TxBuffer[SPI_TX_SIZE];
	uint8_t UART_RxBuffer[UART_RX_SIZE];

	uint16_t 	Power_Shutdown_Pressed;
	uint16_t 	Power_Shutdown_Pressed_Time;
	uint16_t 	ADC_Values[ADC1_CNT];
	uint32_t 	VBUS_mv;
	uint32_t 	PowerStage_mv;
	uint32_t 	PowerStageSetVoltage_mv;

} GlobalDeviceState;

extern struct DeviceState GlobalDeviceState;
extern LPTIM_HandleTypeDef hlptim1;
extern CRC_HandleTypeDef hcrc;

/* USER CODE END ET */

/* Exported constants --------------------------------------------------------*/
/* USER CODE BEGIN EC */

/* USER CODE END EC */

/* Exported macro ------------------------------------------------------------*/
/* USER CODE BEGIN EM */

/* USER CODE END EM */

/* Exported functions prototypes ---------------------------------------------*/
void Error_Handler(void);

/* USER CODE BEGIN EFP */

/* USER CODE END EFP */

/* Private defines -----------------------------------------------------------*/
#define VDDA_APPLI 3300
#define POWER_ON_OFF_SIGNAL_Pin GPIO_PIN_13
#define POWER_ON_OFF_SIGNAL_GPIO_Port GPIOC
#define POWER_ON_OFF_SIGNAL_EXTI_IRQn EXTI13_IRQn
#define ADC_PowerStage_Pin GPIO_PIN_0
#define ADC_PowerStage_GPIO_Port GPIOA
#define ADC_VBUS_Pin GPIO_PIN_1
#define ADC_VBUS_GPIO_Port GPIOA
#define DAC_PowerStage_Pin GPIO_PIN_4
#define DAC_PowerStage_GPIO_Port GPIOA
#define SPI_SCK_Pin GPIO_PIN_5
#define SPI_SCK_GPIO_Port GPIOA
#define SPI_MISO_Pin GPIO_PIN_6
#define SPI_MISO_GPIO_Port GPIOA
#define SPI_MOSI_Pin GPIO_PIN_7
#define SPI_MOSI_GPIO_Port GPIOA
#define SPI_NSS_Pin GPIO_PIN_0
#define SPI_NSS_GPIO_Port GPIOB
#define SPI_NSS_EXTI_IRQn EXTI0_IRQn
#define CHG_INT_Pin GPIO_PIN_1
#define CHG_INT_GPIO_Port GPIOB
#define CHG_INT_EXTI_IRQn EXTI1_IRQn
#define CHG_STAT_Pin GPIO_PIN_2
#define CHG_STAT_GPIO_Port GPIOB
#define SMBC_Pin GPIO_PIN_10
#define SMBC_GPIO_Port GPIOB
#define SMBD_Pin GPIO_PIN_11
#define SMBD_GPIO_Port GPIOB
#define LED_RED_Pin GPIO_PIN_12
#define LED_RED_GPIO_Port GPIOB
#define LED_GREEN_Pin GPIO_PIN_13
#define LED_GREEN_GPIO_Port GPIOB
#define ENABLE_PiPower_Pin GPIO_PIN_8
#define ENABLE_PiPower_GPIO_Port GPIOA
#define ENABLE_PowerStage_Pin GPIO_PIN_10
#define ENABLE_PowerStage_GPIO_Port GPIOA
#define I2C_SCL_Pin GPIO_PIN_6
#define I2C_SCL_GPIO_Port GPIOB
#define I2C_SDA_Pin GPIO_PIN_7
#define I2C_SDA_GPIO_Port GPIOB

/* USER CODE BEGIN Private defines */

/* USER CODE END Private defines */

#ifdef __cplusplus
}
#endif

#endif /* __MAIN_H */
