/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * File Name          : app_freertos.c
  * Description        : Code for freertos applications
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2022 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under Ultimate Liberty license
  * SLA0044, the "License"; You may not use this file except in compliance with
  * the License. You may obtain a copy of the License at:
  *                             www.st.com/SLA0044
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "FreeRTOS.h"
#include "task.h"
#include "main.h"
#include "cmsis_os.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "adc.h"
#include "dac.h"
#include "bq25792.h"
#include "bq40z80.h"
#include "usb_device.h"
#include "usbpd_pwr_user.h"
#include "usart.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */


/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define TRIGGER_GAUGE				(1 << 0)
#define TRIGGER_CHARGER				(1 << 1)
#define TRIGGER_GAUGE_SHUTDOWN		(1 << 2)
#define TRIGGER_SYSTEM_RESET		(1 << 3)
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
/* USER CODE BEGIN Variables */
volatile uint32_t i_SystemState = 0;

/* USER CODE END Variables */
/* Definitions for defaultTask */
osThreadId_t defaultTaskHandle;
const osThreadAttr_t defaultTask_attributes = {
  .name = "defaultTask",
  .priority = (osPriority_t) osPriorityNormal,
  .stack_size = 1024 * 4
};

/* Private function prototypes -----------------------------------------------*/
/* USER CODE BEGIN FunctionPrototypes */
void Set_PowerStageOutputVoltage(uint32_t);
/* USER CODE END FunctionPrototypes */

void StartDefaultTask(void *argument);

void MX_FREERTOS_Init(void); /* (MISRA C 2004 rule 8.1) */

/* USER CODE BEGIN PREPOSTSLEEP */
__weak void PreSleepProcessing(uint32_t ulExpectedIdleTime)
{
/* place for user code */
//	HAL_SuspendTick();
//	HAL_LPTIM_TimeOut_Start_IT(&hlptim1, 0xFFFF, ulExpectedIdleTime);

}

__weak void PostSleepProcessing(uint32_t ulExpectedIdleTime)
{
/* place for user code */
//	HAL_LPTIM_TimeOut_Stop_IT(&hlptim1);
//	HAL_ResumeTick();
}
/* USER CODE END PREPOSTSLEEP */

/**
  * @brief  FreeRTOS initialization
  * @param  None
  * @retval None
  */
void MX_FREERTOS_Init(void) {
  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* USER CODE BEGIN RTOS_MUTEX */
  /* add mutexes, ... */
  /* USER CODE END RTOS_MUTEX */

  /* USER CODE BEGIN RTOS_SEMAPHORES */
  /* add semaphores, ... */
  /* USER CODE END RTOS_SEMAPHORES */

  /* USER CODE BEGIN RTOS_TIMERS */
  /* start timers, add new ones, ... */
  /* USER CODE END RTOS_TIMERS */

  /* USER CODE BEGIN RTOS_QUEUES */
  /* add queues, ... */
  /* USER CODE END RTOS_QUEUES */

  /* Create the thread(s) */
  /* creation of defaultTask */
  defaultTaskHandle = osThreadNew(StartDefaultTask, NULL, &defaultTask_attributes);

  /* USER CODE BEGIN RTOS_THREADS */
  /* add threads, ... */
  /* USER CODE END RTOS_THREADS */

  /* USER CODE BEGIN RTOS_EVENTS */
  /* add events, ... */
  /* USER CODE END RTOS_EVENTS */

}

/* USER CODE BEGIN Header_StartDefaultTask */
/**
  * @brief  Function implementing the defaultTask thread.
  * @param  argument: Not used
  * @retval None
  */
/* USER CODE END Header_StartDefaultTask */
void StartDefaultTask(void *argument)
{
  /* USER CODE BEGIN StartDefaultTask */
	static uint32_t LED_CNT = 0;
  //Check If Powered On From Battery?

  if(GlobalDeviceState.ADC_Values[ADC1_VUSB] <= 300)
  {
	HAL_GPIO_TogglePin(ENABLE_PiPower_GPIO_Port, ENABLE_PiPower_Pin);
  }

  HAL_UART_Receive_DMA(&hlpuart1, GlobalDeviceState.UART_RxBuffer, UART_RX_SIZE);

  HAL_GPIO_TogglePin(LED_GREEN_GPIO_Port, LED_GREEN_Pin);

  /* Infinite loop */
  for(;;)
  {
	if(GlobalDeviceState.Charger->state.chrg_status & Fast_Charge)
	{
		HAL_GPIO_WritePin(LED_GREEN_GPIO_Port, LED_GREEN_Pin, GPIO_PIN_SET);

		if(LED_CNT++ % (GlobalDeviceState.Charger->state.chrg_status == Fast_Charge ? 50 : 100) == 0)
		{
			HAL_GPIO_TogglePin(LED_RED_GPIO_Port, LED_RED_Pin);
		}
	}
	else
	{
		HAL_GPIO_WritePin(LED_RED_GPIO_Port, LED_RED_Pin, GPIO_PIN_SET);
		HAL_GPIO_WritePin(LED_GREEN_GPIO_Port, LED_GREEN_Pin, GPIO_PIN_RESET);
	}

	switch(GlobalDeviceState.Power_Shutdown_Pressed)
	{
		case  0:
			if(GlobalDeviceState.Power_Shutdown_Pressed_Time != 0)
			{
				HAL_GPIO_TogglePin(ENABLE_PiPower_GPIO_Port, ENABLE_PiPower_Pin);
				GlobalDeviceState.Power_Shutdown_Pressed_Time = 0;
			}
			break;
		case 1:
			if(GlobalDeviceState.Power_Shutdown_Pressed_Time++ > 300 /* ~ 3 Seconds */)
			{
				i_SystemState |= TRIGGER_GAUGE_SHUTDOWN;
			}
			break;
	}

	if(i_SystemState & TRIGGER_GAUGE)
	{
		i_SystemState &= ~TRIGGER_GAUGE;
	}
	if(i_SystemState & TRIGGER_CHARGER)
	{
		#ifdef DEBUG
			printf("Charger Request!\n");
		#endif

		if(bq25792_update_state())
		{
			if(
				!GlobalDeviceState.Charger->state.vbat_present
				||
				GlobalDeviceState.Charger->state.chrg_status == Charge_Termination_Done
				||
				GlobalDeviceState.Charger->state.fault_0
				||
				GlobalDeviceState.Charger->state.fault_1
			)
			{
				#ifdef DEBUG
					printf("Stop Charging...\n");
				#endif
				bq25792_set_charging(BQ25792_OFF);
			}
			else if(GlobalDeviceState.Charger->state.vbat_present)
			{
				#ifdef DEBUG
					printf("Start Charging...\n");
				#endif
				bq25792_set_charging(BQ25792_ON);
			}
		}

		i_SystemState &= ~TRIGGER_CHARGER;
	}

	if(i_SystemState & TRIGGER_GAUGE_SHUTDOWN)
	{
		HAL_GPIO_WritePin(ENABLE_PiPower_GPIO_Port, ENABLE_PiPower_Pin, GPIO_PIN_RESET);
		bq40z80_Shutdown(GlobalDeviceState.Gauge);
		i_SystemState &= ~TRIGGER_GAUGE_SHUTDOWN;
	}

	if(i_SystemState & TRIGGER_SYSTEM_RESET)
	{
		i_SystemState &= ~TRIGGER_SYSTEM_RESET;
	}

    GlobalDeviceState.PowerStage_mv = (((330.0f + 100.0f) / 100.0f)) /* Voltage Divider */ * __LL_ADC_CALC_DATA_TO_VOLTAGE( VDDA_APPLI, GlobalDeviceState.ADC_Values[ADC1_PowerStage], LL_ADC_RESOLUTION_12B); /* mV */;

    osDelay(10);

	if(HAL_GPIO_ReadPin(ENABLE_PowerStage_GPIO_Port, ENABLE_PowerStage_Pin))
	{
		float R2 = 100000.f;
		float R3 = 10000.f;
		float RB = 8450.f;
		float VFB = 0.596f;
		float Resolution = 3.3f / 4096;
		float Vout = 0.001f * GlobalDeviceState.PowerStageSetVoltage_mv;
		uint32_t APPROX_DAC_VAL = (uint32_t)((RB * (VFB/R3 - (Vout-VFB) / R2) + VFB) / Resolution);

		uint32_t TMP_DAC_VAL = HAL_DAC_GetValue(&hdac1, DAC_CHANNEL_1);

		int32_t DIFF = (APPROX_DAC_VAL - TMP_DAC_VAL);

		if(abs(DIFF) > 25)
		{
			TMP_DAC_VAL += (DIFF * 0.85);
			HAL_DAC_SetValue(&hdac1, DAC_CHANNEL_1, DAC_ALIGN_12B_R, TMP_DAC_VAL);
		}
		else
		{
			if(GlobalDeviceState.PowerStage_mv > GlobalDeviceState.PowerStageSetVoltage_mv)
			{
				if(TMP_DAC_VAL < 4095 - 1)
				{
					HAL_DAC_SetValue(&hdac1, DAC_CHANNEL_1, DAC_ALIGN_12B_R, ++TMP_DAC_VAL);
				}
			}
			else
			{
				if(TMP_DAC_VAL > 0)
				{
					HAL_DAC_SetValue(&hdac1, DAC_CHANNEL_1, DAC_ALIGN_12B_R, --TMP_DAC_VAL);
				}
			}
		}
	}
  }
  /* USER CODE END StartDefaultTask */
}

/* Private application code --------------------------------------------------*/
/* USER CODE BEGIN Application */

inline void Set_PowerStageOutputVoltage(uint32_t VoltageInMillivolts)
{
	GlobalDeviceState.PowerStageSetVoltage_mv = VoltageInMillivolts;

	float R2 = 100000.f;
	float R3 = 10000.f;
	float RB = 8450.f;
	float VFB = 0.596f;
	float Resolution = 3.3f / 4096;
	float Vout = 0.001f * VoltageInMillivolts; //0.05f * (SPI_Data.Data & 0xFF); //
	uint32_t DAC_VAL = (uint32_t)((RB * (VFB/R3 - (Vout-VFB) / R2) + VFB) / Resolution);

	#ifdef DEBUG
		printf("Set PowerState[%lu]: %lu mV\n", (uint32_t)DAC_VAL, (uint32_t)VoltageInMillivolts);
	#endif

	HAL_DAC_SetValue(&hdac1, DAC_CHANNEL_1, DAC_ALIGN_12B_R, DAC_VAL);
}

void HAL_SPI_RxHalfCpltCallback(SPI_HandleTypeDef *hspi)
{
	struct OptoSense_SPI SPI_Data = GlobalDeviceState.SPI_RxBuffer.Packet;

	#ifdef DEBUG
		printf("Received - BUS: 0x%.2X | CMD/Addr.: 0x%.2X\n", SPI_Data.BUS, SPI_Data.P1.Raw);
	#endif

	switch(SPI_Data.BUS)
	{
		case OptoSense_BUS_Self:
		{
			switch(SPI_Data.P1.Self_CMD)
			{
				case OptoSense_DISABLE_POWERSTAGE:
				{
					HAL_GPIO_WritePin(ENABLE_PowerStage_GPIO_Port, ENABLE_PowerStage_Pin, GPIO_PIN_RESET);
					#ifdef DEBUG
						printf("PowerStage: Off\n");
					#endif
				}
				break;
				case OptoSense_ENABLE_POWERSTAGE:
				{
					HAL_GPIO_WritePin(ENABLE_PowerStage_GPIO_Port, ENABLE_PowerStage_Pin, GPIO_PIN_SET);
					#ifdef DEBUG
						printf("PowerStage: On\n");
					#endif
				}
				break;
				case OptoSense_SET_POWERSTAGE:
				{
					if(SPI_Data.Data < 1)
						break;

					Set_PowerStageOutputVoltage(50.0f * (SPI_Data.Data & 0xFF) /*50mV per step*/);
				}
				break;
				case OptoSense_SystemReset:
				{
					i_SystemState |= TRIGGER_SYSTEM_RESET;
				}
				break;
				case OptoSense_BatteryShutdown:
				{
					HAL_GPIO_WritePin(ENABLE_PiPower_GPIO_Port, ENABLE_PiPower_Pin, GPIO_PIN_RESET);
				}
				break;
			}
		}
		break;
		case OptoSense_BUS_CHARGER:
		{
			HAL_I2C_Mem_Read_DMA(&hi2c1, ADDR_BQ25790, SPI_Data.P1.Register, 1, GlobalDeviceState.SPI_TxBuffer, GlobalDeviceState.SPI_RxBuffer.Packet.Length);
		}
		break;
		case OptoSense_BUS_GAUGE:
		{
			HAL_SMBUS_Master_Transmit_IT(&hsmbus2, ADDR_BQ40Z80, (uint8_t*)&SPI_Data.P1, 1, SMBUS_FIRST_AND_LAST_FRAME_NO_PEC);
		}
		break;
	}
}

void HAL_SPI_RxCpltCallback(SPI_HandleTypeDef *hspi)
{
	HAL_SPI_Transmit_DMA(&hspi1, (uint8_t *)GlobalDeviceState.SPI_TxBuffer, GlobalDeviceState.SPI_RxBuffer.Packet.Length);
}

void HAL_SMBUS_MasterTxCpltCallback(SMBUS_HandleTypeDef *hsmbus)
{
	HAL_SMBUS_Master_Receive_IT(hsmbus, ADDR_BQ40Z80, GlobalDeviceState.SPI_TxBuffer, GlobalDeviceState.SPI_RxBuffer.Packet.Length, SMBUS_FIRST_AND_LAST_FRAME_NO_PEC);
}

void HAL_GPIO_EXTI_Falling_Callback(uint16_t GPIO_Pin)
{
	if(GPIO_Pin == SPI_NSS_Pin)
	{
		if(HAL_GPIO_ReadPin(ENABLE_PiPower_GPIO_Port, ENABLE_PiPower_Pin))
		{
			HAL_SPI_Receive_DMA(&hspi1, (uint8_t *)GlobalDeviceState.SPI_RxBuffer.RAW, SPI_RX_SIZE);
		}
	}
	else if(GPIO_Pin == CHG_INT_Pin)
	{
		i_SystemState |= TRIGGER_CHARGER;
	}
	else if(GPIO_Pin == POWER_ON_OFF_SIGNAL_Pin)
	{
		GlobalDeviceState.Power_Shutdown_Pressed = 0;
	}
}

void HAL_GPIO_EXTI_Rising_Callback(uint16_t GPIO_Pin)
{
	if(GPIO_Pin == SPI_NSS_Pin)
	{
		HAL_SPI_Abort(&hspi1); // We Have To Reset SPI States...
	}
	else if(GPIO_Pin == POWER_ON_OFF_SIGNAL_Pin)
	{
		GlobalDeviceState.Power_Shutdown_Pressed = 1;
	}
}

int32_t BSP_USBPD_PWR_VBUSGetVoltage(uint32_t Instance, uint32_t *pVoltage)
{
  if ((Instance >= USBPD_PWR_INSTANCES_NBR) || (NULL == pVoltage))
    return BSP_ERROR_WRONG_PARAM;

  return *pVoltage = GlobalDeviceState.VBUS_mv = (((3300.0f + 499.0f) / 499.0f)) /* Voltage Divider */ * __LL_ADC_CALC_DATA_TO_VOLTAGE( VDDA_APPLI, GlobalDeviceState.ADC_Values[ADC1_VUSB], LL_ADC_RESOLUTION_12B); /* mV */
}

void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart)
{
#ifdef DEBUG
	printf("UART: %s\n", GlobalDeviceState.UART_RxBuffer);
#endif
}
/* USER CODE END Application */

